<?php
/**
 * @file
 * Provide 3d secure page callbacks for KBC Paypage.
 */

/**
 * Handle 3-D Secure requests (page callback).
 *
 * The 3-D Secure form is base64 and html entity encoded in the url GET
 * parameter 'result'.
 *
 * @return string
 *   The HTML page that is used to ask the user for 3-D Secure verification.
 */
function commerce_kbcpaypage_3d_secure() {
  return str_replace('&#58;', ':', (base64_decode($_GET['result'])));
}

/**
 * Handle 3-D Secure responses (page callback).
 *
 * Check if the 3-D Secure response is valid and update the transaction.
 */
function commerce_kbcpaypage_3d_secure_callback() {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  // Get the feedback information, check if the SHASIGN and ORDERID are present.
  if (!($feedback = _commerce_kbcpaypage_get_feedback()) || empty($feedback['ORDERID'])) {
    _commerce_kbcpaypage_3d_secure_callback_error(FALSE);
  }

  // Load order and payment method.
  $order = commerce_order_load(current(explode('#', $feedback['ORDERID'])));
  $payment_method = commerce_payment_method_instance_load($order->data['payment_method']);

  // Verify SHASIGN.
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.sha1');
  if (_commerce_kbcpaypage_valid_sha1_callback($payment_method, $feedback)) {
    // Process the transaction based on the KBC Paypage parameters received.
    _commerce_kbcpaypage_process_transaction($order, $payment_method, $feedback);
    $payment_status = _commerce_kbcpaypage_feedback_status($feedback['STATUS']);
    if ($payment_status['code'] != COMMERCE_PAYMENT_STATUS_FAILURE) {
      // Activate alias.
      if (!empty($feedback['KBCALIAS'])) {
        module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.cardonfile');
        $alias = explode('-', $feedback['KBCALIAS']);
        _commerce_kbcpaypage_cardonfile_activate($alias[0], $alias[1]);
      }
      drupal_goto(commerce_checkout_order_uri($order));
    }

    // Invalid verification (by customer).
    _commerce_kbcpaypage_3d_secure_callback_error($feedback, '3-D Secure verification failed.', TRUE);
  }

  // SHASIGN verification failed.
  _commerce_kbcpaypage_3d_secure_callback_error($feedback, 'Invalid SHASIGN.');
}

/**
 * Handle 3-D Secure errors.
 *
 * @param array|bool $feedback
 *   3-D Secure feedback information.
 *   FALSE if there is no valid feedback information.
 * @param string $message
 *   (optional) A detailed error message to add to the transaction log.
 * @param bool $usermessage
 *   (optional) Specifies whether the detailed error message or a generic error
 *   message should be displayed to the end user.
 */
function _commerce_kbcpaypage_3d_secure_callback_error(array $feedback, $message = '', $usermessage = FALSE) {
  if (!$feedback) {
    drupal_set_message(t('We encountered an error processing your transaction. Please contact us to resolve the issue.'), 'error');
    drupal_goto('<front>');
  }
  // Load order and payment method.
  $order = commerce_order_load(current(explode('#', $feedback['ORDERID'])));
  $payment_method = commerce_payment_method_instance_load($order->data['payment_method']);

  // Log error response.
  $transaction = commerce_payment_transaction_new('kbcpaypage', $order->order_id);
  $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
  $transaction->instance_id = $payment_method['instance_id'];
  $transaction->amount = $feedback['AMOUNT'];
  $transaction->currency_code = $feedback['CURRENCY'];
  $transaction->message = t('An error occured while processing this transaction using DirectLink and 3D-Secure. @message', array('@message' => $message));
  $transaction->payload[REQUEST_TIME] = $feedback;
  commerce_payment_transaction_save($transaction);

  // Display error detailed or generic error message to the end user.
  if ($usermessage) {
    drupal_set_message(t('@message', array('@message' => $message)), 'error');
  }
  else {
    drupal_set_message(t('We encountered an error processing your transaction. Please contact us to resolve the issue.'), 'error');
  }

  // Update order status to the checkout page, since the review page is not
  // used.
  $order = commerce_order_status_update($order, 'checkout_checkout', FALSE, NULL, 'Customer continued to the previous checkout page (Payment pane is on the checkout page).');

  // Redirect to the checkout page.
  drupal_goto(commerce_checkout_order_uri($order));
}
