<?php
/**
 * @file
 * Provide admin functionalities such as a settings form for KBC Paypage Payment
 * methods and refunding and updating transactions.
 */

/**
 * Update the status of a transaction.
 *
 * @param CommerceOrder $order
 *   The order for which a transaction status needs to be updated.
 * @param CommercePaymentTransaction $transaction
 *   The transaction for which the status needs to be updated.
 */
function commerce_kbcpaypage_transaction_update($order, $transaction) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.directlink');

  // Send an update request using directlink.
  $response = _commerce_kbcpaypage_directlink_transaction_update($transaction);

  // Display an error message if the response is invalid.
  if (!$response || !isset($response['NCRESPONSE']['STATUS']) || $response['NCRESPONSE']['STATUS'] == 0) {
    drupal_set_message(t('The transaction was not updated.'), 'error');
  }
  // Update and save the transaction and display a confirmation message if the
  // response is valid.
  else {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
    $transaction->remote_status = $response['NCRESPONSE']['STATUS'];
    $payment_status = _commerce_kbcpaypage_feedback_status($response['NCRESPONSE']['STATUS']);
    $transaction->status = $payment_status['code'];
    $transaction->message = $payment_status['message'];
    $transaction->payload = $response['NCRESPONSE'];
    commerce_payment_transaction_save($transaction);

    drupal_set_message(t('The transaction was updated.'), 'status');
  }

  // Redirect to the order transaction overview page.
  drupal_goto(t('admin/commerce/orders/@order/payment', array('@order' => $order->order_id)));
}

/**
 * Form constructor to (partially) refund a transaction.
 *
 * @param CommerceOrder $order
 *   The order for which a transaction status needs to be (partially) refunded.
 * @param CommercePaymentTransaction $transaction
 *   The transaction to be (partially) refunded.
 *
 * @see commerce_kbcpaypage_transaction_refund_form_validate()
 * @see commerce_kbcpaypage_transaction_refund_form_submit()
 *
 * @ingroup forms
 */
function commerce_kbcpaypage_transaction_refund_form(array $form, array &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Initialize the default amount and currency.
  $default_amount = 0;
  $default_currency = $transaction->currency_code;

  // Set the totally paid amount for this order (including refunds) as the
  // default value.
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  if ($total = _commerce_kbcpaypage_total_amount_by_order($order)) {
    $default_amount = $total['amount'];
    $default_currency = $total['currency'];
  }
  $default_amount = number_format(commerce_currency_amount_to_decimal($default_amount, $default_currency), 2, '.', '');

  // Retrieve the order balance.
  $balance = commerce_payment_order_balance($order);

  $description = t('Authorization: @amount,', array('@amount' => commerce_currency_format($transaction->amount, $transaction->currency_code))) .
    t('Order balance: @balance', array('@balance' => commerce_currency_format($balance['amount'], $balance['currency_code'])));

  // Add the amount textfield.
  $form['amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Refund amount'),
    '#description' => t('@description', array('@description' => $description)),
    '#default_value' => $default_amount,
    '#field_suffix' => check_plain($transaction->currency_code),
    '#size' => 16,
  );

  // Add the confirmation and cancel buttons.
  $form = confirm_form($form,
      t('What amount do you want to refund?'),
      'admin/commerce/orders/' . $order->order_id . '/payment',
      '',
      t('Refund'),
      t('Cancel'),
      'confirm'
  );

  return $form;
}

/**
 * Form validation handler for commerce_kbcpaypage_payment_transaction_form().
 *
 * @see commerce_kbcpaypage_transaction_refund_form_submit()
 */
function commerce_kbcpaypage_transaction_refund_form_validate(array $form, array $form_state) {
  $amount = $form_state['values']['amount'];

  // Ensure a positive numeric amount has been entered for capture.
  if (!is_numeric($amount) || $amount <= 0) {
    form_set_error('amount', t('You must specify a positive numeric amount to refund.'));
    return FALSE;
  }

  // Ensure the amount is less than or equal to the total paid amount (including
  // refunds).
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  if ($total = _commerce_kbcpaypage_total_amount_by_order($form_state['order'])) {
    if ($amount > commerce_currency_amount_to_decimal($total['amount'], $total['currency'])) {
      form_set_error('amount', t('You refund more than the amount totally paid through KBC Paypage.'));
    }
  }
  else {
    form_set_error('amount', t('The maximum refundable amount cannot be calculated.'));
  }
}

/**
 * Form submission handler for commerce_kbcpaypage_transaction_refund_form().
 *
 * @see commerce_kbcpaypage_transaction_refund_form_validate()
 */
function commerce_kbcpaypage_transaction_refund_form_submit(array $form, array $form_state) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.directlink');

  $order = $form_state['order'];
  $transaction = $form_state['transaction'];

  $amount = commerce_currency_decimal_to_amount($form_state['values']['amount'], $transaction->currency_code);

  // Complete refund.
  if ($transaction->amount == commerce_currency_decimal_to_amount($form_state['values']['amount'], $transaction->currency_code)) {
    $response = _commerce_kbcpaypage_directlink_transaction_refund($transaction);
  }
  // Partial refund.
  else {
    $charge = commerce_currency_decimal_to_amount($form_state['values']['amount'], $transaction->currency_code);
    $response = _commerce_kbcpaypage_directlink_transaction_refund($transaction, $charge);
  }
  // Display an error message if the response is invalid.
  if (!$response || !isset($response['NCRESPONSE']['STATUS']) || $response['NCRESPONSE']['STATUS'] == 0) {
    drupal_set_message(t('The transaction was not refunded.'), 'error');
  }
  // Create and save a new transaction and display a confirmation message if the
  // response is valid.
  else {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
    $new_transaction = commerce_payment_transaction_new('kbcpaypage', $order->order_id);
    $new_transaction->instance_id = $order->data['payment_method'];
    $new_transaction->remote_id = !empty($response['NCRESPONSE']['PAYIDSUB']) ? $response['NCRESPONSE']['PAYID'] . '/' . $response['NCRESPONSE']['PAYIDSUB'] : $response['NCRESPONSE']['PAYID'];
    $new_transaction->amount = commerce_currency_decimal_to_amount((-1) * $response['NCRESPONSE']['AMOUNT'], $response['NCRESPONSE']['CURRENCY']);
    $new_transaction->currency_code = $response['NCRESPONSE']['CURRENCY'];
    $new_transaction->remote_status = $response['NCRESPONSE']['STATUS'];
    $payment_status = _commerce_kbcpaypage_feedback_status($response['NCRESPONSE']['STATUS']);
    $new_transaction->status = $payment_status['code'];
    $new_transaction->message = $payment_status['message'];
    $new_transaction->payload = $response['NCRESPONSE'];

    commerce_payment_transaction_save($new_transaction);

    drupal_set_message(t('Refund was successful.'), 'status');
  }
  drupal_goto(t('admin/commerce/orders/@order/payment', array('@order' => $order->order_id)));
}
