<?php

/**
 * @file
 * KBC Paypage bulk process functionality.
 */

/**
 * Handles generic bulk process requests.
 *
 * @param array $settings
 *   Payment method settings.
 * @param array $request
 *   Details about the request to be processed.
 *
 * @return array
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   Returns FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_bulk_process(array $settings, array $request) {
  // Construct request file.
  if (!isset($request['operation'])) {
    return FALSE;
  }

  $number_of_operations = 0;

  // Send file.
  $params = array(
    'REPLY_TYPE' => 'XML',
    'PROCESS_MODE' => 'SEND',
    'PSPID' => $settings['authentication']['pspid'],
    'USERID' => $settings['directlink']['api_user'],
    'PSWD' => $settings['directlink']['api_pswd'],
  );

  $params['FILE'] = '';

  // Body.
  switch ($request['operation']) {
    case 'DELALIAS':
      if (!isset($request['aliasses']) || empty($request['aliasses'])) {
        return FALSE;
      }
      // DELALIAS;alias;;;;;;
      foreach ($request['aliasses'] as $id => $alias) {
        $params['FILE'] .= 'DELALIAS;'
         . $alias . ';;;;;;'
         . "\n";
      }
      // Use the id of any of the aliasses.
      $params['FILE_REFERENCE'] = 'delalias_' . key($request['aliasses']);
      $number_of_operations = count($request['aliasses']);
      $params['NB_ALIAS'] = $number_of_operations;
      break;
  }

  // Footer.
  $params['FILE'] .= 'OTF;';

  $xml = _commerce_kbcpaypage_bulk_process_curl($settings, $params);

  if (!$xml) {
    return FALSE;
  }

  if (isset($xml->FORMAT_CHECK_ERROR)) {
    return FALSE;
  }

  switch ($request['operation']) {
    case 'ADDALIAS':
    case 'DELALIAS':
      if (!isset($xml->SEND_FILE->SUMMARY->NB_ALIAS) || $xml->SEND_FILE->SUMMARY->NB_ALIAS != $number_of_operations) {
        return FALSE;
      }
      break;
  }

  if (!isset($xml->SEND_FILE->FILEID)) {
    return FALSE;
  }

  $file_id = (string) $xml->SEND_FILE->FILEID;

  // Check file.
  $params = array(
    'REPLY_TYPE' => 'XML',
    'PROCESS_MODE' => 'CHECK',
    'PSPID' => $settings['authentication']['pspid'],
    'USERID' => $settings['directlink']['api_user'],
    'PSWD' => $settings['directlink']['api_pswd'],
    'PFID' => $file_id,
  );

  $xml = _commerce_kbcpaypage_bulk_process_curl($settings, $params);

  if (!$xml) {
    return FALSE;
  }

  switch ($request['operation']) {
    case 'ADDALIAS':
    case 'DELALIAS':
      if (!isset($xml->FORMAT_CHECK->SUMMARY->NB_ALIAS) || $xml->FORMAT_CHECK->SUMMARY->NB_ALIAS != $number_of_operations) {
        return FALSE;
      }
      break;
  }

  if (isset($xml->NCERROR)) {
    watchdog('commerce_kbcpaypage', 'cURL error: @error', array('@error' => 'Please log into the Paypage back office to see the details of the error.'), WATCHDOG_ERROR);
    return FALSE;
  }

  // Process file.
  $params = array(
    'REPLY_TYPE' => 'XML',
    'PROCESS_MODE' => 'PROCESS',
    'PSPID' => $settings['authentication']['pspid'],
    'USERID' => $settings['directlink']['api_user'],
    'PSWD' => $settings['directlink']['api_pswd'],
    'PFID' => $file_id,
  );

  $xml = _commerce_kbcpaypage_bulk_process_curl($settings, $params);

  if (!$xml) {
    return FALSE;
  }

  switch ($request['operation']) {
    case 'ADDALIAS':
    case 'DELALIAS':
      if (!isset($xml->PROCESSING->SUMMARY->NB_ALIAS) || $xml->PROCESSING->SUMMARY->NB_ALIAS != $number_of_operations) {
        return FALSE;
      }
      break;
  }

  if (isset($xml->NCERROR)) {
    watchdog('commerce_kbcpaypage', 'cURL error: @error', array('@error' => 'Please log into the Paypage back office to see the details of the error.'), WATCHDOG_ERROR);
    return FALSE;
  }

  return TRUE;
}

/**
 * Helper function for _commerce_kbcpaypage_bulk_process.
 *
 * This function handles the actual curl requests.
 *
 * @param array $settings
 *   Payment method settings.
 * @param array $params
 *   Request parameters.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   Returns FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_bulk_process_curl(array $settings, array $params) {
  // Construct url.
  $url = variable_get('commerce_kbcpaypage_provider_url', 'https://secure.paypage.be/ncol/');
  $url .= $settings['authentication']['account'] . '/AFU_agree.asp';

  // Construct and do curl request.
  $curl = curl_init();
  curl_setopt($curl, CURLOPT_URL, $url);
  curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($curl, CURLOPT_POST, FALSE);
  curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($params));
  $response = curl_exec($curl);

  // Parse answer.
  $xml = simplexml_load_string($response, "SimpleXMLElement", LIBXML_NOCDATA);

  // Log any errors to the watchdog.
  if ($error = curl_error($curl)) {
    watchdog('commerce_kbcpaypage', 'cURL error: @error', array('@error' => $error), WATCHDOG_ERROR);
    return FALSE;
  }
  curl_close($curl);

  return $xml;
}
