<?php

/**
 * @file
 * KBC Paypage Credit Card Alias functionality using commerce_cardonfile.
 */

/**
 * Creates an instance to store partial credit card information locally.
 *
 * @param int $uid
 *   The id of the user for whom to store the credit card information.
 * @param array $payment_method
 *   The payment method used with this credit card.
 * @param array $pane_values
 *   The values submitted by the user on the payment pane.
 * @param int $status
 *   The status of the created cardonfile instance.
 */
function _commerce_kbcpaypage_cardonfile_create($uid, array $payment_method, array $pane_values, $status = 1) {
  // Payment option.
  $payment_option = explode('|', $pane_values['payment_options']);

  $alias = $uid . '-' . substr($pane_values['credit_card']['number'], -4);

  // Check if a card with this number already exists for this user.
  $cards = _commerce_kbcpaypage_cardonfile_load_multiple_by_uid($uid);
  foreach ($cards as $card) {
    if ($card->remote_id == $alias) {
      $card_data = $card;
      break;
    }
  }

  // Create a new cardonfile if this number does not exist for this user.
  if (empty($card_data)) {
    $card_data = commerce_cardonfile_new();
  }

  // Set card data.
  $card_data->uid = $uid;
  $card_data->status = $status;
  $card_data->payment_method = $payment_method['method_id'];
  $card_data->instance_id = $payment_method['instance_id'];
  $card_data->remote_id = $alias;
  $card_data->card_type = $pane_values['payment_options'];
  $card_data->card_name = $pane_values['credit_card']['owner'];
  $card_data->card_number = substr($pane_values['credit_card']['number'], -4);
  $card_data->card_exp_month = $pane_values['credit_card']['exp_month'];
  $card_data->card_exp_year = $pane_values['credit_card']['exp_year'];

  if (!empty($pane_values['cardonfile_instance_default'])) {
    $card_data->instance_default = 1;
  }

  // Save and log the creation of the new card on file.
  commerce_cardonfile_save($card_data);
}

/**
 * Activates a cardonfile instance.
 *
 * @param int $uid
 *   The id of the user for whom to activate the cardonfile instance.
 * @param string $card_number
 *   Partial card number of the partially stored credit card information.
 */
function _commerce_kbcpaypage_cardonfile_activate($uid, $card_number) {
  $cards = _commerce_kbcpaypage_cardonfile_load_multiple_by_uid($uid);
  foreach ($cards as $card) {
    if ($card->card_number == $card_number) {
      $card_data = $card;
      break;
    }
  }
  if (!empty($card_data)) {
    $card_data->status = 1;
    commerce_cardonfile_save($card_data);
    return TRUE;
  }

  return FALSE;
}

/**
 * Load a cardonfile instance and set as default if requested.
 *
 * @param array $pane_values
 *   The values submitted by the user on the payment pane.
 *   This array will be completed with the alias and card type of the selected
 *   cardonfile instance.
 *
 * @return bool
 *   Specifies whether the cardonfile instance exists or not.
 */
function _commerce_kbcpaypage_cardonfile_load(array &$pane_values) {
  $card_data = commerce_cardonfile_load($pane_values['cardonfile']);

  // Set default and save if requested.
  if (empty($card_data->instance_default) && !empty($pane_values['instance_default'])) {
    $card_data->instance_default = 1;
    commerce_cardonfile_save($card_data);
  }

  // Fail now if it is no longer available or the card is inactive.
  if (empty($card_data) || $card_data->status == 0) {
    drupal_set_message(t('The requested card on file is no longer valid.'), 'error');
    return FALSE;
  }

  $pane_values['alias'] = $card_data->remote_id;
  $pane_values['payment_options'] = $card_data->card_type;

  return TRUE;
}

/**
 * Load all active and passive cardonfile instances linked to a specific user.
 *
 * @param int $uid
 *   The id of the user for whom to load the cardonfile instances.
 *
 * @return array
 *   An array of cardonfile instances indexed by their id.
 */
function _commerce_kbcpaypage_cardonfile_load_multiple_by_uid($uid) {
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'commerce_cardonfile')
    ->propertyCondition('uid', $uid);
  $result = $query->execute();
  if (isset($result['commerce_cardonfile'])) {
    return commerce_cardonfile_load_multiple(array_keys($result['commerce_cardonfile']));
  }
  else {
    return array();
  }
}
