<?php
/**
 * @file
 * Provide commerce_payment callbacks for KBC Paypage.
 */

/**
 * Implements hook_form_FORM_ID_alter().
 */
function commerce_kbcpaypage_form_rules_ui_edit_element_alter(array &$form, array $form_state) {
  // Define custom validate and submit handlers for the commerce_payment
  // settings form callback.
  if (isset($form['parameter']['payment_method'])) {
    $payment_method = $form['parameter']['payment_method']['settings']['payment_method']['method_id']['#value'];
    if ($payment_method == 'kbcpaypage' || $payment_method == 'kbcpaypage_one_page_checkout') {
      $form['#validate'][] = 'commerce_kbcpaypage_settings_form_validate';
      $form['#submit'][] = 'commerce_kbcpaypage_settings_form_submit';
    }
  }
}

/**
 * Implements CALLBACK_commerce_payment_method_settings_form().
 *
 * @param array $settings
 *   (optional) The current settings.
 *
 * @return array
 *   An array containing form elements for the payment method's settings form
 *   included as part of the payment method's enabling action in Rules.
 *
 * @see commerce_kbcpaypage_settings_form_validate()
 * @see commerce_kbcpaypage_settings_form_submit()
 */
function commerce_kbcpaypage_settings_form(array $settings = NULL) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  // Check whether the settings form is loaded for the kbcpaypage payment
  // method or the kbcpaypage_one_page_checkout payment method.
  $one_page = strpos(current_path(), 'kbcpaypage_one_page_checkout') > 0;

  // Initialize common default settings.
  $default_settings = array(
    'authentication' => array(
      'account' => 'test',
      'pspid' => '',
      'sha_in' => '',
      'sha_out' => '',
    ),
    'directlink' => array(
      'api_user' => '',
      'api_pswd' => '',
      'sha_in' => '',
    ),
    'currency' => variable_get('commerce_default_currency', 'USD'),
    'language' => '',
    'language_list' => array(),
    'payment_methods' => array(
      'enabled' => array(),
    ),
    'payment_template'  => array(
      'setting' => FALSE,
      'title' => '',
    ),
    'ccalias' => array(
      'setting' => FALSE,
    ),
  );

  // Initialize additional default settings for kbcpaypage_one_page_checkout.
  if ($one_page) {
    $default_settings['3d_secure'] = array(
      'setting' => FALSE,
    );
  }
  // Initialize additional default settings for kbcpaypage.
  else {
    $default_settings['predefined'] = array(
      'setting' => FALSE,
    );
    $default_settings['ccalias']['usage'] = '';
  }

  // Use the default settings if current settings are not available.
  $settings += $default_settings;

  $form = array();
  $languages = language_list();
  $language_codes = _commerce_kbcpaypage_get_language_code_list();

  // Paypage authentication settings.
  $form['authentication'] = array(
    '#type' => 'fieldset',
    '#title' => t('KBC Paypage authentication'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['authentication']['account'] = array(
    '#type' => 'radios',
    '#title' => t('KBC Paypage account'),
    '#options' => array(
      'test' => ('Test account - for testing purposes only'),
      'prod' => ('Production account - use for processing real transactions'),
    ),
    '#default_value' => $settings['authentication']['account'],
  );
  $form['authentication']['pspid'] = array(
    '#type' => 'textfield',
    '#title' => t('PSPID'),
    '#description' => t('Your KBC Paypage PSPID login username'),
    '#default_value' => $settings['authentication']['pspid'],
    '#required' => TRUE,
  );
  $form['authentication']['sha_in'] = array(
    '#type' => 'textfield',
    '#title' => t('SHA-IN Pass phrase'),
    '#description' => t('The SHA-IN Pass phrase as entered in KBC Paypage technical settings - "Data and origin verification" tab.'),
    '#default_value' => $settings['authentication']['sha_in'],
    '#required' => !$one_page,
  );
  $form['authentication']['sha_out'] = array(
    '#type' => 'textfield',
    '#title' => t('SHA-1-OUT Pass phrase'),
    '#description' => t('The SHA-1-OUT Pass phrase as entered in KBC Paypage technical settings - "Transaction feedback" tab.'),
    '#default_value' => $settings['authentication']['sha_out'],
    '#required' => !$one_page,
  );

  // DirectLink authentication settings.
  $form['directlink'] = array(
    '#type' => 'fieldset',
    '#description' => t('Server to server communication is used for features as one page checkout, updating payment information and refunding payments. A KBC Paypage PRO account is required for these features.'),
    '#title' => t('KBC Paypage server to server authentication'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['directlink']['api_user'] = array(
    '#default_value' => $settings['directlink']['api_user'],
    '#required' => FALSE,
    '#title' => t('API User'),
    '#type' => 'textfield',
    '#required' => $one_page,
  );
  $form['directlink']['api_pswd'] = array(
    '#default_value' => $settings['directlink']['api_pswd'],
    '#required' => FALSE,
    '#title' => t('API User password'),
    '#type' => 'textfield',
    '#required' => $one_page,
  );
  $form['directlink']['sha_in'] = array(
    '#default_value' => $settings['directlink']['sha_in'],
    '#required' => FALSE,
    '#title' => t('DirectLink SHA-IN Pass phrase as entered in KBC Paypage technical settings - "Data and origin verification" tab.'),
    '#type' => 'textfield',
  );

  $form['currency'] = array(
    '#type' => 'select',
    '#title' => t('Currency code'),
    '#description' => t('Transactions can only be processed in one of the listed currencies.'),
    '#options' => _commerce_kbcpaypage_currencies(),
    '#default_value' => $settings['currency'],
  );
  $form['language_list'] = array(
    '#type' => 'fieldset',
    '#title' => t('Language codes'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $default_language_code = (isset($settings['language_list']['default_language'])) ? $settings['language_list']['default_language'] : 'en_US';
  foreach ($languages as $language) {
    // Attempt to provide an intelligent default based on the language prefix.
    // E.g., language prefix 'fr' would create the default KBC Paypage
    // language 'fr_FR'.
    $default_prefix_code = explode('-', $language->prefix);
    $default_prefix_code = $default_prefix_code[0] . '_' . drupal_strtoupper($default_prefix_code[0]);
    // Verify whether the intelligent default value is valid. If not, fall back
    // to default language code.
    $default_prefix_code = (in_array($default_prefix_code, array_keys($language_codes))) ? $default_prefix_code : $default_language_code;

    $form['language_list'][$language->language] = array(
      '#type' => 'select',
      '#title' => check_plain($language->name),
      '#options' => $language_codes,
      '#default_value' => (isset($settings['language_list'][$language->language])) ? $settings['language_list'][$language->language] : $default_prefix_code,
      '#description' => t('Language code. For instance: en_US, nl_NL, fr_FR, ...'),
    );
  }
  $form['language_list']['default_language'] = array(
    '#type' => 'select',
    '#title' => 'Default',
    '#options' => $language_codes,
    '#default_value' => $default_language_code,
    '#description' => t('Language code. For instance: en_US, nl_NL, fr_FR, ...'),
  );

  // Predefine the payment option on your site.
  // For one page checkout, the payment option is allways predefined.
  if (!$one_page) {
    $form['predefined'] = array(
      '#type' => 'fieldset',
      '#title' => t('Predefine selection'),
      '#description' => t('Selecting this will allow users to select the payment option on your site.'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['predefined']['setting'] = array(
      '#type' => 'checkbox',
      '#title' => t('Predefine the payment on your site'),
      '#default_value' => $settings['predefined']['setting'],
    );
  }

  // Enable payment options.
  $form['payment_methods'] = array(
    '#type' => 'fieldset',
    '#title' => t('Payment options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  // Load payment methods.
  $payment_methods = _commerce_kbcpaypage_payment_options();

  // For one page checkout, remove non credit card payment methods.
  if ($one_page) {
    foreach ($payment_methods as $key => $value) {
      $key_split = explode('|', $key);
      if ($key_split[0] != 'CreditCard') {
        unset($payment_methods[$key]);
      }
    }
  }

  // Add icons to the payment methods.
  foreach ($payment_methods as &$payment_method) {
    $payment_method = _commerce_kbcpaypage_icon($payment_method) . '<span class="commerce-kbcpaypage-icon-title">' . $payment_method['name'] . '</span>';
  }

  $form['payment_methods']['enabled'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Enabled payment methods'),
    '#description' => t('This is only a setting for the icons displayed on the checkout form. The payment methods required need to be enabled and configured in the KBC Paypage back office.'),
    '#options' => $payment_methods,
    '#default_value' => $settings['payment_methods']['enabled'],
  );

  // Let users use a dynamic template.
  $form['payment_template'] = array(
    '#type' => 'fieldset',
    '#title' => t('Dynamic template'),
    '#description' => t('Use the predefined dynamic template of KBC. You need to enable dynamic templates in the back office of the KBC Paypage.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['payment_template']['setting'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use KBC Paypage dynamic template'),
    '#default_value' => $settings['payment_template']['setting'],
  );
  $form['payment_template']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Dynamic template title'),
    '#default_value' => $settings['payment_template']['title'],
    '#states' => array(
      'visible' => array(
        ':input[name="parameter[payment_method][settings][payment_method][settings][payment_template][setting]"]' => array(
          'checked' => TRUE,
        ),
      ),
    ),
  );

  // Enable 3-D Secure (only for one page checkout).
  if ($one_page) {
    $form['3d_secure'] = array(
      '#type' => 'fieldset',
      '#title' => t('3-D Secure'),
      '#description' => t('3-D Secure offers a high level of security, as it allows customers to be identified unambiguously through technologies, e.g., html passwords, Digipass, card readers, biometrics, etc., implemented by the issuing banks.'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['3d_secure']['setting'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use 3-D Secure'),
      '#default_value' => $settings['3d_secure']['setting'],
    );
  }

  // Save credit card aliasses.
  $form['ccalias'] = array(
    '#type' => 'fieldset',
    '#title' => t('Credit card aliasses'),
    '#description' => t("Selecting this will save credit card aliasses, so returning users have to enter their credit card information only once. Only the credit card alias is saved in the database of the webshop, the credit card information itself is securely saved on the KBC Paypage server. A KBC Paypage PRO account is required for this feature."),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['ccalias']['setting'] = array(
    '#type' => 'checkbox',
    '#title' => t('Save credit card aliasses'),
    '#default_value' => $settings['ccalias']['setting'],
  );

  if (!$one_page) {
    $form['ccalias']['usage'] = array(
      '#type' => 'textfield',
      '#title' => t('Reason for the alias registration.'),
      '#description' => t('A text explaining the reason for the Alias registration. This text will be displayed to the end user above the checkbox to approve/refuse the storage of his/her financial profile.'),
      '#default_value' => $settings['ccalias']['usage'],
      '#states' => array(
        'visible' => array(
          ':input[name="parameter[payment_method][settings][payment_method][settings][ccalias][setting]"]' => array(
            'checked' => TRUE,
          ),
        ),
      ),
    );
  }

  // Define cardonfile setting, that will be used by the commerce_cardonfile
  // module. The value will be copied during settings_form_submit.
  $form['cardonfile'] = array(
    '#type' => 'hidden',
  );

  return $form;
}

/**
 * Form validation handler for CALLBACK_commerce_payment_method_settings_form().
 *
 * @see commerce_kbcpaypage_settings_form_submit()
 */
function commerce_kbcpaypage_settings_form_validate(array $form, array &$form_state) {
  $one_page = $form_state['values']['parameter']['payment_method']['settings']['payment_method']['method_id'] == 'kbcpaypage_one_page_checkout';
  $settings = $form_state['values']['parameter']['payment_method']['settings']['payment_method']['settings'];
  $form_element_prefix = 'parameter][payment_method][settings][payment_method][settings][';

  // Check if a title is defined if dynamic template is enabled.
  if (isset($settings['payment_template']['setting']) && !empty($settings['payment_template']['setting'])) {
    if (!isset($settings['payment_template']['title']) || empty($settings['payment_template']['title'])) {
      form_set_error($form_element_prefix . 'payment_template][title', t('You need to define a title for your dynamic template.'));
    }
  }

  if ((isset($settings['directlink']['api_user']) && !empty($settings['directlink']['api_user']))
    || (isset($settings['directlink']['api_pswd']) && !empty($settings['directlink']['api_pswd']))
    || (isset($settings['directlink']['sha_in']) && !empty($settings['directlink']['sha_in']))
    || (isset($settings['ccalias']['setting']) && !empty($settings['ccalias']['setting']))
    || (isset($settings['3d_secure']['setting']) && !empty($settings['3d_secure']['setting']))
    ) {
    drupal_set_message(t('Notice: A KBC Paypage PRO account is required for some of the selected features.'), 'warning');
  }

  // Copy alias setting to cardonfile, since this setting is used by the
  // commerce_cardonfile module.
  $form_state['rules_element']->settings['payment_method']['settings']['cardonfile'] = $settings['ccalias']['setting'];
}

/**
 * Form submit handler for CALLBACK_commerce_payment_method_settings_form().
 *
 * @see commerce_kbcpaypage_settings_form_validate()
 */
function commerce_kbcpaypage_settings_form_submit(array $form, array $form_state) {
  $one_page = $form_state['values']['parameter']['payment_method']['settings']['payment_method']['method_id'] == 'kbcpaypage_one_page_checkout';
  $settings = $form_state['values']['parameter']['payment_method']['settings']['payment_method']['settings'];

  // Generate the dynamic template if enabled.
  if ($settings['payment_template']['setting'] == 1) {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
    $title = $settings['payment_template']['title'];

    $file = _commerce_kbcpaypage_create_template($title, $one_page);
    if ($file) {
      $location = file_create_url($file->uri);
      drupal_set_message(t('We have generated your file in this location: @location', array('@location' => $location)));
    }
    else {
      drupal_set_message(t('We could not generate your file. Please check the permissions of your public directory'), 'error');
    }
  }
}

/**
 * Get a list of enabled currencies.
 *
 * @return array
 *   List with all currecies enabled in commerce.
 *
 * @TODO: figure out which currencies KBC Paypage actually supports
 */
function _commerce_kbcpaypage_currencies() {
  // Build a currency options list from all enabled currencies.
  $options = array();

  foreach (commerce_currencies(TRUE) as $currency_code => $currency) {
    $options[$currency_code] = t(
      '@code - !name',
      array(
        '@code' => $currency['code'],
        '@symbol' => $currency['symbol'],
        '!name' => $currency['name'],
      )
    );

    if (!empty($currency['symbol'])) {
      $options[$currency_code] .= ' - ' . check_plain($currency['symbol']);
    }
  }

  return $options;
}

/**
 * Implements CALLBACK_commerce_payment_method_submit_form().
 */
function commerce_kbcpaypage_submit_form(array $payment_method, array $pane_values, array $checkout_pane, $order) {
  if ($payment_method['method_id'] == 'kbcpaypage_one_page_checkout' || ($payment_method['method_id'] == 'kbcpaypage' && !empty($payment_method['settings']['predefined']['setting']))) {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
    $payment_options = _commerce_kbcpaypage_payment_options($payment_method);
    foreach ($payment_options as &$payment_option) {
      $payment_option = _commerce_kbcpaypage_icon($payment_option, TRUE);
    }
    $path = drupal_get_path('module', 'commerce_kbcpaypage') . '/css/';
    $form['payment_options_' . $payment_method['method_id']] = array(
      '#type' => 'radios',
      '#title' => t('Select your payment option'),
      '#options' => $payment_options,
      '#default_value' => current(array_keys($payment_options)),
      '#required' => TRUE,
      '#attached' => array(
        'css' => array(
          $path . 'commerce_kbcpaypage_detail.css',
        ),
      ),
    );

    // Add credit card form if one page is enabled for the selected payment
    // method.
    if ($payment_method['method_id'] == 'kbcpaypage_one_page_checkout') {
      // Load credit card module.
      module_load_include('inc', 'commerce_payment', 'includes/commerce_payment.credit_card');

      // Prepare the fields to include on the credit card form.
      $fields = array(
        'code' => '',
        'owner' => '',
      );

      // Add credit card form.
      $credit_card_form = commerce_payment_credit_card_form($fields);
      $form['credit_card'] = $credit_card_form['credit_card'];
    }

    return $form;
  }
}

/**
 * Implements CALLBACK_commerce_payment_method_submit_form_validate().
 */
function commerce_kbcpaypage_submit_form_validate(array $payment_method, array $pane_form, array $pane_values, $order, array $form_parents = array()) {
  // No Credit Card validation needed if using stored Card data.
  if (!empty($payment_method['settings']['cardonfile']) && !empty($pane_values['cardonfile']) && $pane_values['cardonfile'] != 'new') {
    return TRUE;
  }
  if (isset($pane_values['credit_card'])) {
    module_load_include('inc', 'commerce_payment', 'includes/commerce_payment.credit_card');
    // Validate the credit card fields.
    $settings = array(
      'form_parents' => array_merge($form_parents, array('credit_card')),
    );

    if (!commerce_payment_credit_card_validate($pane_values['credit_card'], $settings)) {
      return FALSE;
    }
  }
}

/**
 * Implements CALLBACK_commerce_payment_method_submit_form_submit().
 */
function commerce_kbcpaypage_submit_form_submit(array $payment_method, array $pane_form, array &$pane_values, $order, array $charge) {
  // Set payment_options.
  if (isset($pane_values['payment_options_kbcpaypage'])) {
    $pane_values['payment_options'] = $pane_values['payment_options_kbcpaypage'];
  }
  if (isset($pane_values['payment_options_kbcpaypage_one_page_checkout'])) {
    $pane_values['payment_options'] = $pane_values['payment_options_kbcpaypage_one_page_checkout'];
  }

  // Save payment_option in order.
  if (!empty($payment_method['settings']['predefined']['setting'])) {
    $order->data['payment_option'] = $pane_values['payment_options'];
  }

  // Aliases.
  if (!empty($payment_method['settings']['cardonfile']) && isset($pane_values['cardonfile'])) {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.cardonfile');
    // Save cardonfile data.
    if ($pane_values['cardonfile'] == 'new') {
      _commerce_kbcpaypage_cardonfile_create($order->uid, $payment_method, $pane_values, 0);
    }
    // Load cardonfile data (and save default).
    else {
      // Try to load card data. If this fails, do not submit.
      if (!(_commerce_kbcpaypage_cardonfile_load($pane_values))) {
        return FALSE;
      }
    }
  }

  // Process one page checkout payment.
  if (isset($pane_values['credit_card'])) {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.directlink');
    if ($response = _commerce_kbcpaypage_directlink_single_payment($payment_method, $pane_values, $order, $charge)) {
      // Skip the review page.
      $order = commerce_order_status_update($order, 'checkout_payment', FALSE, NULL, 'Customer continued to the next checkout page via a DirectLink Payment.');

      // Check if we received no error.
      if ($response['NCRESPONSE']['NCERROR'] == '0') {

        // Check if we received a request for 3-D Secure.
        if ($response['NCRESPONSE']['STATUS'] == '46') {
          if (empty($response['HTML_ANSWER'])) {
            drupal_set_message(t('We encountered an error processing your transaction. Please contact us to resolve the issue.'), 'error');
            return FALSE;
          }
          drupal_goto('commerce_kbcpaypage/3d_secure', array(
            'query' => array(
              'result' => $response['HTML_ANSWER']),
          ));
        }

        module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
        _commerce_kbcpaypage_process_transaction($order, $payment_method, $response['NCRESPONSE']);
        $payment_status = _commerce_kbcpaypage_feedback_status($response['NCRESPONSE']['STATUS']);
        if ($payment_status['code'] != COMMERCE_PAYMENT_STATUS_FAILURE) {
          // Set card on file on active.
          if (!empty($payment_method['settings']['cardonfile']) && isset($pane_values['cardonfile']) && $pane_values['cardonfile'] == 'new') {
            _commerce_kbcpaypage_cardonfile_activate($order->uid, substr($pane_values['credit_card']['number'], -4));
          }
          // Redirect to complete page.
          drupal_goto(commerce_checkout_order_uri($order));
          return;
        }
      }

      // Log error response.
      $transaction = commerce_payment_transaction_new('kbcpaypage', $order->order_id);
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->instance_id = $payment_method['instance_id'];
      $transaction->amount = $charge['amount'];
      $transaction->currency_code = $charge['currency_code'];
      $transaction->message = t('An error occured while processing this transaction using DirectLink.');
      $transaction->payload[REQUEST_TIME] = $response;
      commerce_payment_transaction_save($transaction);

      $order = commerce_order_status_update($order, 'checkout_checkout', FALSE, NULL, 'Customer continued to the previous checkout page (Payment pane is on the checkout page).');

      drupal_set_message(t('Payment failed with error code: @code and message: @message', array(
        '@code' => $response['NCRESPONSE']['NCERROR'],
        '@message' => $response['NCRESPONSE']['NCERRORPLUS']
        )),
        'error'
      );
      return FALSE;
    }

    $order = commerce_order_status_update($order, 'checkout_checkout', FALSE, NULL, 'Customer continued to the previous checkout page (Payment pane is on the checkout page).');
    drupal_set_message(t('We encountered an error processing your transaction. Please contact us to resolve the issue.'), 'error');
    return FALSE;
  }

  // Set PM and BRAND in $_SESSION.
  if (isset($pane_values['payment_options'])) {
    $selected = explode('|', $pane_values['payment_options']);

    $_SESSION['kbcpaypage_selected'] = array(
      'PM' => $selected[0],
      'BRAND' => $selected[1],
    );
  }
}

/**
 * Implements CALLBACK_commerce_payment_method_redirect_form().
 *
 * Returns form elements that should be submitted to the redirected payment
 * service.
 */
function commerce_kbcpaypage_redirect_form(array $form, array $form_state, $order, array $payment_method) {
  // Display an error if the enabling action's settings haven't been configured.
  if (empty($payment_method['settings']['authentication']['pspid'])) {
    drupal_set_message(t('KBC Paypage is not configured for use. No KBC Paypage PSPID has been specified.'), 'error');
    return array();
  }

  // Set feedback URLs.
  $settings = array(
    // Return to the previous page when payment is canceled.
    'cancel_return' => url('checkout/' . $order->order_id . '/payment/back/' . $order->data['payment_redirect_key'], array('absolute' => TRUE)),

    // Return to the payment redirect page for processing successful payments.
    'return' => url('checkout/' . $order->order_id . '/payment/return/' . $order->data['payment_redirect_key'], array('absolute' => TRUE)),

    // Specify the current payment method instance ID in the notify_url.
    'payment_method' => $payment_method['instance_id'],
  );

  return _commerce_kbcpaypage_build_redirect_form($form, $order, $payment_method['settings'] + $settings);
}

/**
 * Helper function for commerce_kbcpaypage_redirect_form.
 *
 * Builds a KBC Paypage payment form from an order object.
 */
function _commerce_kbcpaypage_build_redirect_form($form, $order, $settings) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  $wrapper = entity_metadata_wrapper('commerce_order', $order);

  // Get financial info.
  $currency_code = $wrapper->commerce_order_total->currency_code->value();
  $amount = $wrapper->commerce_order_total->amount->value();
  if (!empty($wrapper->commerce_customer_billing->value()->commerce_customer_address)) {
    $customer_name = $wrapper->commerce_customer_billing->commerce_customer_address->name_line->value();
  }
  else {
    $customer_name = '';
  }

  // Build the data array that will be translated into hidden form values.
  // NOTE: keys should be uppercase for the SHA-1 security string.
  $data = array(
    // General parameters.
    'PSPID' => $settings['authentication']['pspid'],
    'ORDERID' => _commerce_kbcpaypage_get_unique_orderid($order),
    'AMOUNT' => $amount,
    'CURRENCY' => $currency_code,
    'LANGUAGE' => _commerce_kbcpaypage_get_language_code($settings),
    'CN' => $customer_name,
    'EMAIL' => $order->mail,
    'TXTCOLOR' => '#777777',

    // Layout.
    'TITLE' => t(
      'Order @order_number at @store',
      array(
        '@order_number' => $order->order_number,
        '@store' => variable_get('site_name', url('<front>', array('absolute' => TRUE))),
      )
    ),
    'FONTTYPE' => 'Verdana',
    'BUTTONBGCOLOR' => '#1aa0e1',
    'BUTTONTXTCOLOR' => '#FFFFFF',

    // Feedback URLs.
    'ACCEPTURL' => $settings['return'],
    'DECLINEURL' => $settings['return'],
    'EXCEPTIONURL' => $settings['return'],
    'CANCELURL' => $settings['cancel_return'],
  );

  // Add the credit card alias values when this has been enabled and a user is
  // logged in.
  if (!empty($settings['ccalias']['setting']) && !empty($order->uid)) {
    $alias = $order->uid;
    // Add the selected PM and BRAND when predefined is enabled.
    if (!empty($settings['predefined']['setting']) && isset($_SESSION['kbcpaypage_selected'])) {
      $alias .= '-' . $_SESSION['kbcpaypage_selected']['PM'] . '-' . $_SESSION['kbcpaypage_selected']['BRAND'];
    }
    $data['ALIAS'] = $alias;
    $data['ALIASUSAGE'] = $settings['ccalias']['usage'];
  }

  // Add the predefined settings when this has been enabled.
  if (!empty($settings['predefined']['setting'])) {
    // Add the selected PM and BRAND to the data array.
    if (isset($_SESSION['kbcpaypage_selected'])) {
      $data['PM'] = $_SESSION['kbcpaypage_selected']['PM'];
      $data['BRAND'] = $_SESSION['kbcpaypage_selected']['BRAND'];
      unset($_SESSION['kbcpaypage_selected']);
    }
  }

  // Add the dynamic template when this is enabled.
  if (isset($settings['payment_template']['setting']) && $settings['payment_template']['setting'] == 1) {
    $template = 'public://kbc-dynamic-template.html';
    if (!file_exists($template)) {
      module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
      // Try to recreate the file.
      _commerce_kbcpaypage_create_template($settings['payment_template']['title']);
    }
    $template = file_create_url($template);
    $data['TP'] = $template;
  }

  // Allow modules to add or alter the data sent to KBC Paypage before signing.
  drupal_alter('commerce_kbcpaypage_data', $data, $order, $settings);

  // NOTE: data should be ordered alphabetically for the SHA-1 string.
  ksort($data);

  // Build the KBC Paypage redirection URL.
  $url = variable_get('commerce_kbcpaypage_provider_url', 'https://secure.paypage.be/ncol/');
  $url .= $settings['authentication']['account'] . '/orderstandard_utf8.asp';
  $form['#action'] = $url;

  // Generate SHA-1 security string & add hidden parameters to form.
  $sha1_sig = '';
  $sha1suffix = $settings['authentication']['sha_in'];
  foreach ($data as $name => $value) {
    if (!empty($value)) {
      $value = trim($value);
      // Build the SHA-1 string from all filled in parameters.
      $sha1_sig .= $name . '=' . $value . $sha1suffix;
      // Add the value as a hidden form element.
      $form[$name] = array('#type' => 'hidden', '#value' => $value);
    }
  }
  // Add the calculated SHA-1 string.
  $form['SHASign'] = array('#type' => 'hidden', '#value' => sha1($sha1_sig));

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Proceed with payment'),
  );
  return $form;
}

/**
 * Implements CALLBACK_commerce_payment_method_redirect_form_validate().
 */
function commerce_kbcpaypage_redirect_form_validate($order, array $payment_method) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  // Process callback information from KBC Paypage.
  return _commerce_kbcpaypage_process_callback($order, $payment_method);
}

/**
 * Implements CALLBACK_commerce_payment_method_redirect_form_back().
 */
function commerce_kbcpaypage_redirect_form_back($order, array $payment_method) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  // Display a message indicating the customer cancelled the payment.
  drupal_set_message(t('You have cancelled the payment at KBC Paypage but may resume the checkout process when you are ready.'));

  // Remove the payment information from the order data array.
  unset($order->data['payment_method']);

  // If Payment pane is on the checkout page, redirect to the checkout page.
  $checkout_panes = commerce_checkout_panes();
  if ($checkout_panes['commerce_payment']['page'] == 'checkout') {
    $order = commerce_order_status_update($order, 'checkout_checkout', FALSE, NULL, 'Customer continued to the previous checkout page (Payment pane is on the checkout page).');
  }

  // Process callback information from KBC Paypage.
  return _commerce_kbcpaypage_process_callback($order, $payment_method);
}

/**
 * Menu callback handler for KBC Paypage Direct HTTP server-to-server requests.
 */
function commerce_kbcpaypage_callback() {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  // Load order and payment_method objects.
  if ($feedback = _commerce_kbcpaypage_get_feedback()) {
    $order = commerce_order_load(current(explode('#', $feedback['ORDERID'])));
    if ($order) {
      // Load the payment method, since it won't be available if no $order
      // object was passed in as argument.
      $payment_method = commerce_payment_method_instance_load($order->data['payment_method']);
      if ($payment_method && $payment_method['method_id'] == 'kbcpaypage') {
        module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.sha1');
        if (_commerce_kbcpaypage_valid_sha1_callback($payment_method, $feedback)) {
          module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
          _commerce_kbcpaypage_process_transaction($order, $payment_method, $feedback);
        }
      }
    }
  }
}
