<?php
/**
 * @file
 * Provide KBC Paypage core functionalities.
 */

/**
 * Returns available payment options.
 *
 * If a payment method is provided, only the payment options that are enabled
 * for this payment method are returned.
 *
 * @param array $payment_method
 *   (optional) A payment method instance.
 *
 * @return array
 *   An array containing the available payment options.
 *   Each payment option is represented by a key consisting of a KBC Paypage
 *   payment method and a brand name, seperated by a pipe symbol, and a value
 *   consisting of an array containing an icon name and a human readable name.
 */
function _commerce_kbcpaypage_payment_options(array $payment_method = NULL) {
  $payment_options = array(
    'KBC Online|KBC Online' => array(
      'icon' => 'kbconline',
      'name' => 'KBC Online',
    ),
    'CBC Online|CBC Online' => array(
      'icon' => 'cbconline',
      'name' => 'CBC Online',
    ),
    'CreditCard|BCMC' => array(
      'icon' => 'bancontact',
      'name' => 'Bancontact/Mister Cash',
    ),
    'CreditCard|VISA' => array(
      'icon' => 'visa',
      'name' => 'Visa',
    ),
    'CreditCard|MasterCard' => array(
      'icon' => 'mastercard',
      'name' => 'Mastercard',
    ),
    'CreditCard|American Express' => array(
      'icon' => 'americanexpress',
      'name' => 'American Express',
    ),
    'CreditCard|Diners Club' => array(
      'icon' => 'dinersclub',
      'name' => 'Diners Club',
    ),
    'Belfius Direct Net|Belfius Direct Net' => array(
      'icon' => 'belfiusdirectnet',
      'name' => 'Belfius Direct Net',
    ),
    'giropay|giropay' => array(
      'icon' => 'giropay',
      'name' => 'Giropay',
    ),
    'iDEAL|iDEAL' => array(
      'icon' => 'ideal',
      'name' => 'iDEAL',
    ),
    'ING HomePay|ING HomePay' => array(
      'icon' => 'inghomepay',
      'name' => 'ING HomePay',
    ),
    'Bank transfer BE|Bank transfer BE' => array(
      'icon' => 'banktransfer',
      'name' => 'Bank transfer BE',
    ),
  );

  if (isset($payment_method)) {
    foreach ($payment_options as $key => $value) {
      if (empty($payment_method['settings']['payment_methods']['enabled'][$key])) {
        unset($payment_options[$key]);
      }
    }
  }

  return $payment_options;
}

/**
 * Get a payment status and message based on KBC Paypage status ID.
 *
 * @param int $status
 *   Received KBC Paypage status ID.
 *
 * @return array
 *   Key 'code' contains the commerce status.
 *   Key 'message' contains a textual representation of the KBC Paypage status
 *   ID.
 */
function _commerce_kbcpaypage_feedback_status($status) {
  switch ($status) {
    // SUCCESS.
    // Order stored.
    case 4:
      $st = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $msg = t('Order stored');
      break;

    // Authorized.
    case 5:
      $st = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $msg = t('Authorized');
      break;

    // Refund.
    case 8:
      $st = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $msg = t('Refund');
      break;

    // Refund processed by merchant.
    case 85:
      $st = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $msg = t('Refund processed by merchant');
      break;

    // Payment requested.
    case 9:
      $st = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $msg = t('Payment requested');
      break;

    // Payment processed by merchant.
    case 95:
      $st = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $msg = t('Payment processed by merchant');
      break;

    // PENDING.
    // Stored waiting external result.
    case 40:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Stored waiting external result');
      break;

    // Waiting client payment.
    case 41:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Waiting client payment');
      break;

    // Authorized waiting external result.
    case 50:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Authorized waiting external result');
      break;

    // Authorization waiting.
    case 51:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Authorization waiting');
      break;

    // Stand-by.
    case 55:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Stand-by');
      break;

    // OK with scheduled payments.
    case 56:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('OK with scheduled payments');
      break;

    // Authoriz. to get manually.
    case 59:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Authoriz. to get manually');
      break;

    // Refund pending.
    case 81:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Refund pending');
      break;

    // Refund uncertain.
    case 82:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Refund uncertain');
      break;

    // Payment processing.
    case 91:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Payment processing');
      break;

    // Payment uncertain.
    case 92:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Payment uncertain');
      break;

    // Being processed.
    case 99:
      $st = COMMERCE_PAYMENT_STATUS_PENDING;
      $msg = t('Being processed');
      break;

    // FAILURE.
    // Incomplete or invalid.
    case 0:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Incomplete or invalid');
      break;

    // Cancelled by client.
    case 1:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Cancelled by client');
      break;

    // Authorization refused.
    case 2:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Authorization refused');
      break;

    // Authorization not known.
    case 52:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Authorization not known');
      break;

    // Error in scheduled payments.
    case 57:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Error in scheduled payments');
      break;

    // Authorized and cancelled.
    case 6:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Authorized and cancelled');
      break;

    // Author. deletion waiting.
    case 61:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Author. deletion waiting');
      break;

    // Author. deletion uncertain.
    case 62:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Author. deletion uncertain');
      break;

    // Author. deletion refused.
    case 63:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Author. deletion refused');
      break;

    // Authorized and cancelled.
    case 64:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Authorized and cancelled');
      break;

    // Payment deleted.
    case 7:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment deleted');
      break;

    // Payment deletion pending.
    case 71:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment deletion pending');
      break;

    // Payment deletion uncertain.
    case 72:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment deletion uncertain');
      break;

    // Payment deletion refused.
    case 73:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment deletion refused');
      break;

    // Payment deleted.
    case 74:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment deleted');
      break;

    // Deletion processed by merchant.
    case 75:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Deletion processed by merchant');
      break;

    // Refund refused.
    case 83:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Refund refused');
      break;

    // Payment declined by the acquirer.
    case 84:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment declined by the acquirer');
      break;

    // Payment refused.
    case 93:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Payment refused');
      break;

    // Refund declined by the acquirer.
    case 94:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Refund declined by the acquirer');
      break;

    default:
      $st = COMMERCE_PAYMENT_STATUS_FAILURE;
      $msg = t('Unknown feedback status');
      break;
  }
  return array(
    'code' => $st,
    'message' => $msg,
  );
}

/**
 * Returns a list of supported KBC Paypage language codes.
 *
 * @return array
 *   Available language codes.
 */
function _commerce_kbcpaypage_get_language_code_list() {
  $language_codes = array(
    'ar_AR' => t('Arabic'),
    'cs_CZ' => t('Czech'),
    'zh_CN' => t('Chinese'),
    'da_DK' => t('Danish'),
    'nl_BE' => t('Dutch (Belgium)'),
    'nl_NL' => t('Dutch'),
    'en_GB' => t('English (UK)'),
    'en_US' => t('English (US)'),
    'fr_FR' => t('French'),
    'de_DE' => t('German'),
    'el_GR' => t('Greek'),
    'hu_HU' => t('Hungarian'),
    'it_IT' => t('Italian'),
    'ja_JP' => t('Japanese'),
    'no_NO' => t('Norwegian'),
    'pl_PL' => t('Polish'),
    'pt_PT' => t('Portuguese'),
    'ru_RU' => t('Russian'),
    'sk_SK' => t('Slovak'),
    'es_ES' => t('Spanish'),
    'se_SE' => t('Swedish'),
    'tr_TR' => t('Turkish'),
  );
  return $language_codes;
}

/**
 * Looks up the language code of a enabled language.
 *
 * @param array $settings
 *   Received settings.
 *
 * @return string
 *   Language code.
 */
function _commerce_kbcpaypage_get_language_code(array $settings) {
  global $language;
  $kbc_language_codes = _commerce_kbcpaypage_get_language_code_list();
  if (isset($settings['language_list'][$language->language])) {
    $langcode = $settings['language_list'][$language->language];
  }
  else {
    $default_prefix_code = explode('-', $language->prefix);
    $default_prefix_code = $default_prefix_code[0] . '_' . drupal_strtoupper($default_prefix_code[0]);
    if (isset($kbc_language_codes[$default_prefix_code])) {
      $langcode = $default_prefix_code;
    }
    else {
      $langcode = $settings['language_list']['default_language'];
    }
  }
  return $langcode;
}

/**
 * Create a payment method icon.
 *
 * @param array $payment_method
 *   An array containing the name and icon name for a payment method.
 * @param bool $large
 *   (optional) Indicates whether the constructed icon should be a large icon.
 *
 * @return string
 *   An HTML string representing the payment method icon.
 */
function _commerce_kbcpaypage_icon(array $payment_method, $large = FALSE) {
  $path = drupal_get_path('module', 'commerce_kbcpaypage') . '/images/';
  $variables = array(
    'path' => $path . $payment_method['icon'] . ($large ? '-large' : '') . '.png',
    'title' => $payment_method['name'],
    'alt' => $payment_method['name'],
    'attributes' => array(
      'class' => array('commerce-kbcpaypage' . ($large ? '-large' : '') . '-icon'),
    ),
  );
  return theme('image', $variables);
}

/**
 * Creates a template html file for the dynamic template.
 *
 * @param string $title
 *   A title to display on the dynamic template.
 * @param bool $one_page
 *   Specifies whether the template needs to be created for the kbcpaypage
 *   method or for the kbcpaypage_one_page_checkout payment method.
 *
 * @return File|FALSE
 *   A file object.
 *   FALSE on error.
 */
function _commerce_kbcpaypage_create_template($title = '', $one_page = FALSE) {
  $path = drupal_get_path('module', 'commerce_kbcpaypage');

  $css_file = DRUPAL_ROOT . '/' . $path . '/css/commerce_kbcpaypage_template.css';

  $css_content = file_get_contents($css_file);

  $variables = array(
    'path' => $path . '/images/logo-large.png',
    'title' => 'Logo',
    'alt' => 'Logo',
    'attributes' => array(
      'class' => array('commerce-kbcpaypage-logo'),
    ),
  );
  $logo = theme('image', $variables);

  $output = theme(
    'commerce_kbcpaypage_dynamic_template',
    array(
      'title' => t('@title', array('@title' => $title)),
      'logo' => $logo,
      'css' => $css_content,
    )
  );

  $url = $one_page ? 'public://kbc-dynamic-template-one-page.html' : 'public://kbc-dynamic-template.html';

  return file_save_data($output, $url, FILE_EXISTS_REPLACE);
}

/**
 * Calculate the totally paid amount for a specific orderid.
 *
 * This amount includes payments in success status and refunds in succes or
 * pending status.
 *
 * @param CommerceOrder $order
 *   The order for which to obtain the totally paid amount.
 *
 * @return array|bool
 *   An array with the totally paid amount and corresponding currency or FALSE
 *  if the totally paid amount cannot be calculated.
 */
function _commerce_kbcpaypage_total_amount_by_order($order) {
  // Retrieve all transactions related tot this orderid.
  $query = new EntityFieldQuery();
  $result = $query
    ->entityCondition('entity_type', 'commerce_payment_transaction')
    ->propertyCondition('order_id', $order->order_id)
    ->propertyOrderBy('transaction_id', 'DESC')
    ->execute();

  if (!isset($result['commerce_payment_transaction'])) {
    return FALSE;
  }

  // Initialize total amount and currency.
  $total = 0;
  $currency = '';

  foreach ($result['commerce_payment_transaction'] as $transaction_min) {
    $transaction = commerce_payment_transaction_load($transaction_min->transaction_id);

    // Skip failed transactions.
    if ($transaction->status == COMMERCE_PAYMENT_STATUS_FAILURE) {
      continue;
    }

    // Skip payment pending transactions.
    if (in_array($transaction->remote_status, array(91, 92, 99))) {
      continue;
    }

    // Multi-currency is currently not supported.
    if (!empty($currency) && $currency != $transaction->currency_code) {
      return FALSE;
    }

    if (empty($currency)) {
      $currency = $transaction->currency_code;
    }
    $total += $transaction->amount;
  }

  return array(
    'amount' => $total,
    'currency' => $currency,
  );
}

/**
 * Returns a unique ORDERID for sending to KBC Paypage gateway.
 *
 * KBC Paypage will show an error if we send them a payment request with the
 * ORDERID value that was already sent before (regardless of whether it
 * succeeded or failed). So if there already exist any previously created
 * transaction for this order, let's load them and modify current ORDERID to
 * make sure we send them a unique value.
 *
 * @param CommerceOrder $order
 *   The loaded order that is being processed.
 *
 * @return string
 *   Unique ORDERID value to be sent to the KBC Paypage gateway.
 */
function _commerce_kbcpaypage_get_unique_orderid($order) {
  // Very first payment request will have ORDERID without any modifications, ie
  // 48.
  $unique_orderid = $order->order_number;
  // If any payment transaction already exist, it would need to be modified to
  // be unique.
  if ($transactions = commerce_payment_transaction_load_multiple(array(), array('order_id' => $order->order_id))) {
    $existing_ids = array();
    foreach ($transactions as $transaction) {
      if (!empty($transaction->payload['ORDERID'])) {
        $existing_ids[] = $transaction->payload['ORDERID'];
      }
    }
    $modifier = 1;
    while (in_array($unique_orderid, $existing_ids)) {
      $modifier++;
      // We'll use values like 48#2, 48#3 etc.
      $unique_orderid = $order->order_number . '#' . $modifier;
    }
  }

  return $unique_orderid;
}

/**
 * Gets the KBC Paypage feedback from GET / POST parameters.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback taken from the
 *   $_GET and $_POST superglobals, excluding 'q' and with the parameter keys
 *   uppercased.
 *   FALSE if the SHASIGN parameter is missing (indicating missing or invalid
 *   KBC Paypage feedback).
 */
function _commerce_kbcpaypage_get_feedback() {
  // Initialize feedback.
  $feedback = FALSE;

  if (isset($_REQUEST['SHASIGN'])) {
    // Prepare the feedback values sent by KBC Paypage for processing. We don't
    // use $_REQUEST since this includes the $_SESSION variables.
    $feedback = $_GET + $_POST;

    // Remove the Drupal path from the parameter list.
    unset($feedback['q']);

    // Make sure all feedback keys are uppercase.
    $feedback = array_change_key_case($feedback, CASE_UPPER);
  }

  return $feedback;
}

/**
 * Process callback information from KBC Paypage.
 *
 * This can either be through a redirect after payment, or a Direct HTTP
 * server-to-server request.
 *
 * @param CommerceOrder $order
 *   The order for which the feedback information must be validated.
 * @param array $payment_method
 *   The payment method settings.
 *
 * @return bool
 *   Specifies whether the feedback received from the KBC Paypage is valid.
 */
function _commerce_kbcpaypage_process_callback($order, array $payment_method) {
  if ($feedback = _commerce_kbcpaypage_get_feedback()) {
    module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.sha1');
    if (_commerce_kbcpaypage_valid_sha1_callback($payment_method, $feedback)) {
      // Process the transaction based on the KBC Paypage parameters received.
      _commerce_kbcpaypage_process_transaction($order, $payment_method, $feedback);
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Process the payment transaction with the info received from KBC Paypage.
 *
 * @param CommerceOrder $order
 *   The loaded order that is being processed.
 * @param array $payment_method
 *   The payment method settings.
 * @param array $feedback
 *   The parameters received from KBC Paypage regarding the payment.
 * @param bool $redirect
 *   Specifies whether to call redirect functions or not.
 */
function _commerce_kbcpaypage_process_transaction($order, array $payment_method, array $feedback, $redirect = TRUE) {
  $transaction_id = _commerce_kbcpaypage_get_payment_transaction($feedback);
  if (!$transaction_id) {
    $transaction = commerce_payment_transaction_new('kbcpaypage', $order->order_id);
  }
  else {
    $transaction = commerce_payment_transaction_load($transaction_id);
  }

  $payment_status = _commerce_kbcpaypage_feedback_status($feedback['STATUS']);

  // Identifying data.
  $transaction->instance_id = $payment_method['instance_id'];
  $transaction->remote_id = $feedback['PAYID'];
  // Payment amount details.
  if (isset($feedback['CURRENCY'])){
    $transaction->currency_code = $feedback['CURRENCY'];
  }
  else{
    $transaction->currency_code = 'EUR';
  }
  $transaction->amount = commerce_currency_decimal_to_amount($feedback['AMOUNT'], $feedback['CURRENCY']);
  // Payment status.
  $transaction->remote_status = $feedback['STATUS'];
  $transaction->status = $payment_status['code'];
  $transaction->message = $payment_status['message'];
  $transaction->payload = $feedback;
  commerce_payment_transaction_save($transaction);
  if ($redirect) {
    if ($payment_status['code'] == COMMERCE_PAYMENT_STATUS_FAILURE) {
      commerce_payment_redirect_pane_previous_page($order);
    }
    else {
      commerce_payment_redirect_pane_next_page($order);
    }
  }
}

/**
 * Get the transaction id of the transaction with a specific KBC Paypage PAYID.
 *
 * @param array $feedback
 *   The parameters received from KBC Paypage regarding the payment.
 *
 * @return int
 *   The id of the transaction corresponding with the given KBC Paypage PAYID.
 *   Returns false if no corresponding transaction could be found.
 */
function _commerce_kbcpaypage_get_payment_transaction(array $feedback) {
  $feedback_remote_id = $feedback['PAYID'];
  $query = new EntityFieldQuery();

  $result = $query
    ->entityCondition('entity_type', 'commerce_payment_transaction')
    ->propertyCondition('payment_method', 'kbcpaypage')
    ->propertyCondition('remote_id', $feedback_remote_id)
    ->execute();
  if (isset($result['commerce_payment_transaction']) && count($result['commerce_payment_transaction']) > 0) {
    $transaction = array_pop($result['commerce_payment_transaction']);
    return $transaction->transaction_id;
  }
  return FALSE;
}
