<?php

/**
 * @file
 * KBC Paypage directlink functionality.
 */

/**
 * Handles requests for single payments using DirectLink.
 *
 * @param array $payment_method
 *   Details about the payment method to be used.
 * @param array $pane_values
 *   Submitted values from the checkout page, containing credit card
 *   information.
 * @param CommerceOrder $order
 *   The order to be paid.
 * @param array $charge
 *   The amount to be charged and the corresponding currency.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_directlink_single_payment(array $payment_method, array $pane_values, $order, array $charge) {
  module_load_include('inc', 'commerce_kbcpaypage', 'includes/commerce_kbcpaypage.core');
  $wrapper = entity_metadata_wrapper('commerce_order', $order);
  $settings = $payment_method['settings'];

  // Obtain MMYY format.
  $expirity_date = $pane_values['credit_card']['exp_month'] . substr($pane_values['credit_card']['exp_year'], 2);

  // Payment option.
  $payment_option = explode('|', $pane_values['payment_options']);

  $params = array(
    'ORDERID' => _commerce_kbcpaypage_get_unique_orderid($order),
    'AMOUNT' => $charge['amount'],
    'CURRENCY' => $charge['currency_code'],
    'EMAIL' => $order->mail,
    'OWNERADDRESS' => $wrapper->commerce_customer_billing->commerce_customer_address->thoroughfare->value(),
    'OWNERZIP' => $wrapper->commerce_customer_billing->commerce_customer_address->postal_code->value(),
    'OWNERTOWN' => $wrapper->commerce_customer_billing->commerce_customer_address->locality->value(),
    'OWNERCTY' => $wrapper->commerce_customer_billing->commerce_customer_address->country->value(),
    'OPERATION' => 'SAL',
  );

  // Add Credit Card data if not using an alias.
  if (empty($payment_method['settings']['cardonfile']) || empty($pane_values['cardonfile']) || $pane_values['cardonfile'] == 'new') {
    $params['CARDNO'] = $pane_values['credit_card']['number'];
    $params['ED'] = $expirity_date;
    $params['CN'] = $pane_values['credit_card']['owner'];
    $params['CVC'] = $pane_values['credit_card']['code'];
    $params['PM'] = $payment_option[0];
    $params['BRAND'] = $payment_option[1];
  }

  // Add additional parameters for 3-D Secure.
  if (!empty($settings['3d_secure']['setting'])) {
    global $base_root;
    $callback_url = $base_root . '/commerce_kbcpaypage/3d_secure/callback';
    $params['FLAG3D'] = 'Y';
    $params['WIN3DS'] = 'MAINW';
    $billing_data['HTTP_ACCEPT'] = $_SERVER['HTTP_ACCEPT'];
    $billing_data['HTTP_USER_AGENT'] = $_SERVER['HTTP_USER_AGENT'];
    $params['ACCEPTURL'] = $callback_url;
    $params['DECLINEURL'] = $callback_url;
    $params['EXCEPTIONURL'] = $callback_url;
    $params['LANGUAGE'] = _commerce_kbcpaypage_get_language_code($settings);

    // Add the dynamic template when this is enabled.
    if (!empty($settings['payment_template']['setting'])) {
      $template = 'public://kbc-dynamic-template-one-page.html';
      if (!file_exists($template)) {
        // Try to recreate the file.
        _commerce_kbcpaypage_create_template($settings['payment_template']['title'], TRUE);
      }
      $template = file_create_url($template);
      $params['TP'] = $template;
    }
  }

  // Add additional parameters for Alias Manager.
  if (!empty($payment_method['settings']['cardonfile']) && !empty($pane_values['cardonfile'])) {
    if ($pane_values['cardonfile'] == 'new') {
      $params['ALIAS'] = $order->uid . '-' . substr($pane_values['credit_card']['number'], -4);
      $params['PARAMPLUS'] = 'KBCALIAS=' . $params['ALIAS'];
    }
    else {
      $params['ALIAS'] = $pane_values['alias'];
      $params['CVC'] = $pane_values['cvc'];
    }
  }

  return _commerce_kbcpaypage_directlink_request($settings, $params, 'direct');
}

/**
 * Handles requests for transaction status updates.
 *
 * @param CommercePaymentTransaction $transaction
 *   The transaction for which to update the status.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_directlink_transaction_update($transaction) {
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $settings = $payment_method['settings'];
  $pay_id = explode('/', $transaction->remote_id);
  $params = array(
    'PAYID' => $pay_id[0],
  );
  if (!empty($pay_id[1])) {
    $params['PAYIDSUB'] = $pay_id[1];
  }

  return _commerce_kbcpaypage_directlink_request($settings, $params, 'query');
}

/**
 * Handles requests for transaction status (partial) refunds.
 *
 * @param CommercePaymentTransaction $transaction
 *   The transaction for to process a refund.
 * @param int $amount
 *   (optional) The amount to be refunded.
 *   If the amount is omitted, the complete transaction amount will be refunded.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_directlink_transaction_refund($transaction, $amount = 0) {
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $settings = $payment_method['settings'];
  $pay_id = explode('/', $transaction->remote_id);
  $params = array(
    'OPERATION' => 'RFD',
    'AMOUNT' => empty($amount) ? $transaction->amount : $amount,
    'CURRENCY' => $transaction->currency_code,
    'PAYID' => $pay_id[0],
  );
  if (!empty($pay_id[1])) {
    $params['PAYIDSUB'] = $payid[1];
  }

  return _commerce_kbcpaypage_directlink_request($settings, $params, 'maintenance');
}

/**
 * Handles generic DirectLink requests.
 *
 * @param array $settings
 *   Payment method settings.
 * @param array $params
 *   Details about the request to be processed.
 * @param string $operation
 *   Operation that needs to be processed.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_directlink_request(array $settings, array $params, $operation) {
  // Add credentials to the parameters.
  $params['PSPID'] = $settings['authentication']['pspid'];
  $params['USERID'] = $settings['directlink']['api_user'];
  $params['PSWD'] = $settings['directlink']['api_pswd'];

  // Calculate and add SHASIGN.
  if (!empty($settings['directlink']['sha_in'])) {
    $shasign = '';
    ksort($params);

    foreach ($params as $key => $value) {
      $shasign .= $key . '=' . $value . $settings['directlink']['sha_in'];
    }

    $params['SHASIGN'] = sha1($shasign);
  }

  $url = _commerce_kbcpaypage_directlink_url($settings, $operation);

  // Construct and do curl request.
  $curl = curl_init();
  curl_setopt($curl, CURLOPT_URL, $url);
  curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($curl, CURLOPT_POST, FALSE);
  curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($params));
  $response = curl_exec($curl);

  // Log any errors to the watchdog.
  if ($error = curl_error($curl)) {
    watchdog('commerce_kbcpaypage', 'cURL error: @error', array('@error' => $error), WATCHDOG_ERROR);
    return FALSE;
  }
  curl_close($curl);

  // Parse answer.
  $parser = xml_parser_create();
  xml_parse_into_struct($parser, $response, $values);

  $answer = array();

  foreach ($values as $value) {
    if ($value['tag'] == 'NCRESPONSE' && ($value['type'] == 'open' || $value['type'] == 'complete')) {
      $answer[$value['tag']] = $value['attributes'];
    }
    elseif ($value['tag'] == 'HTML_ANSWER') {
      $answer[$value['tag']] = $value['value'];
    }
  }

  return $answer;
}

/**
 * Generates the url to which a DirectLink request must be sent.
 *
 * @param array $settings
 *   Payment method settings.
 * @param string $operation
 *   Operation that needs to be processed.
 *
 * @return array|bool
 *   An associative array containing the KBC Paypage feedback with the parameter
 *   keys uppercased.
 *   FALSE if there was a problem during the DirectLink request. The
 *   corresponding error is logged using watchdog.
 */
function _commerce_kbcpaypage_directlink_url(array $settings, $operation) {
  // Base url.
  $url = variable_get('commerce_kbcpaypage_provider_url', 'https://secure.paypage.be/ncol/');

  // Account type.
  $url .= $settings['authentication']['account'];

  // Operation.
  switch ($operation) {
    case 'direct':
      $url .= '/orderdirect_utf8.asp';
      break;

    case 'query':
      $url .= '/querydirect_utf8.asp';
      break;

    case 'maintenance':
      $url .= '/maintenancedirect.asp';
      break;
  }

  return $url;
}
