<?php

/**
 * @file
 * SHA-1 security hash specific functionality.
 */

/**
 * Check if SHA1 in callback feedback is valid.
 *
 * @param array $payment_method
 *   The payment method settings.
 * @param array $feedback
 *   The parameters received from KBC Paypage regarding the payment.
 *
 * @return bool
 *   Specifies whether the feedback is valid.
 */
function _commerce_kbcpaypage_valid_sha1_callback(array $payment_method, array $feedback) {
  // Check if the received SHA-1 string is valid for the selected
  // payment_method.
  $sha1out = $payment_method['settings']['authentication']['sha_out'];
  if (!_commerce_kbcpaypage_feedback_valid_sha1($feedback, $sha1out)) {
    watchdog('commerce_kbcpaypage', "SHA signature doesn't match", array(), WATCHDOG_ERROR);
    return FALSE;
  }
  return TRUE;
}

/**
 * Validate the feedback with the SHA-1 signature.
 *
 * @param array $feedback
 *   The feedback information that needs to be checked.
 * @param string $sha1out
 *   The salt that is used as a secret in the calculation of the salted hash.
 *
 * @return bool
 *   Boolean indicating the validity of the feedback information.
 */
function _commerce_kbcpaypage_feedback_valid_sha1(array $feedback, $sha1out) {
  $incoming_sha1 = '';
  foreach (_commerce_kbcpaypage_sha1_out_parameters() as $param) {
    if (isset($feedback[$param]) && $feedback[$param] !== '') {
      $incoming_sha1 .= $param . '=' . $feedback[$param] . $sha1out;
    }
  }

  return strtoupper(sha1($incoming_sha1)) === $feedback['SHASIGN'];
}

/**
 * Returns a list of valid SHA-1 out parameters.
 *
 * See "17.2 SHA-OUT" in "e-Commerce Advanced - Technical Integration Guide for
 * e-Commerce v.5.4.0" for more information.
 *
 * @return array
 *   An array of parameter names to take into account when calculating the
 *   SHA1-OUT security hash.
 */
function _commerce_kbcpaypage_sha1_out_parameters() {
  return array(
    'AAVADDRESS', 'AAVCHECK', 'AAVZIP', 'ACCEPTANCE', 'ALIAS', 'AMOUNT',
    'BIN', 'BRAND', 'CARDNO', 'CCCTY', 'CN', 'COMPLUS', 'CREATION_STATUS',
    'CURRENCY', 'CVC', 'CVCCHECK', 'DCC_COMMPERCENTAGE', 'DCC_CONVAMOUNT',
    'DCC_CONVCCY', 'DCC_EXCHRATE', 'DCC_EXCHRATESOURCE', 'DCC_EXCHRATETS',
    'DCC_INDICATOR', 'DCC_MARGINPERCENTAGE', 'DCC_VALIDHOURS', 'DIGESTCARDNO',
    'ECI', 'ED', 'ENCCARDNO', 'IP', 'IPCTY', 'NBREMAILUSAGE', 'NBRIPUSAGE',
    'NBRIPUSAGE_ALLTX', 'NBRUSAGE', 'NCERROR', 'NCERRORCARDNO', 'NCERRORCN',
    'NCERRORCVC', 'NCERRORED', 'ORDERID', 'PAYID', 'PM', 'SCO_CATEGORY',
    'SCORING', 'STATUS', 'SUBBRAND', 'SUBSCRIPTION_ID', 'TRXDATE', 'VC',
  );
}
